@@ -2,7 +2,7 @@
 #'
 #' Flexible nonparametric modeling of covariates for continuous, binary,
 #' categorical and time-to-event outcomes.
-#' 
+#'
 #' @param K if provided, then coarsen the times of survival responses per the
 #'   quantiles \eqn{1/K, 2/K, ..., K/K} to reduce computational burdern.
 #' @param sparse logical indicating whether to perform variable selection based
@@ -40,20 +40,19 @@
 #' @param keepevery interval at which to keep posterior draws.
 #' @param printevery interval at which to print MCMC progress.
 #'
-#' @details 
+#' @details
 #' \describe{
 #'   \item{Response Types:}{\code{factor}, \code{numeric}, \code{Surv}}
 #' }
-#' 
+#'
 #' Default values for the \code{NULL} arguments and further model details can be
 #' found in the source links below.
-#' 
+#'
 #' @return \code{MLModel} class object.
-#' 
+#'
 #' @seealso \code{\link[BART]{gbart}}, \code{\link[BART]{mbart}},
-#' \code{\link[BART]{surv.bart}}, \code{\link{fit}}, \code{\link{resample}},
-#' \code{\link{tune}}
-#' 
+#' \code{\link[BART]{surv.bart}}, \code{\link{fit}}, \code{\link{resample}}
+#'
 #' @examples
 #' \donttest{
 #' fit(sale_amount ~ ., data = ICHomes, model = BARTModel)
@@ -67,7 +66,7 @@ BARTModel <- function(K = NULL, sparse = FALSE, theta = 0, omega = 1,
                       tau.num = NULL, offset = NULL,
                       ntree = NULL, numcut = 100, ndpost = 1000, nskip = NULL,
                       keepevery = NULL, printevery = 1000) {
-  
+
   MLModel(
     name = "BARTModel",
     label = "Bayesian Additive Regression Trees",
@@ -80,7 +79,7 @@ BARTModel <- function(K = NULL, sparse = FALSE, theta = 0, omega = 1,
       x <- model.matrix(data, intercept = FALSE)
       y <- response(data)
       switch_class(y,
-                   "factor" = {
+                   factor = {
                      assert_equal_weights(weights)
                      if (nlevels(y) == 2) {
                        f <- BART::gbart
@@ -91,12 +90,12 @@ BARTModel <- function(K = NULL, sparse = FALSE, theta = 0, omega = 1,
                      }
                      f(x.train = x, y.train = y, type = "pbart", ...)
                    },
-                   "numeric" = {
+                   numeric = {
                      BART::gbart(x.train = x, y.train = y, w = weights,
                                  sigest = sigest, sigdf = sigdf,
                                  sigquant = sigquant, lambda = lambda, ...)
                    },
-                   "Surv" = {
+                   Surv = {
                     assert_equal_weights(weights)
                     BART::surv.bart(x.train = x, times = y[, "time"],
                                     delta = y[, "status"], K = K, ...)
@@ -121,7 +120,7 @@ BARTModel <- function(K = NULL, sparse = FALSE, theta = 0, omega = 1,
       }
     }
   )
-  
+
 }
 
 MLModelFunction(BARTModel) <- NULL
