@@ -1,8 +1,8 @@
 #' Gradient Boosting with Linear Models
-#' 
+#'
 #' Gradient boosting for optimizing arbitrary loss functions where
 #' component-wise linear models are utilized as base-learners.
-#' 
+#'
 #' @param family optional \code{\link[mboost]{Family}} object.  Set
 #'   automatically according to the class type of the response variable.
 #' @param mstop number of initial boosting iterations.
@@ -13,7 +13,7 @@
 #'   internally when the out-of-bag risk increases at a subsequent iteration.
 #' @param trace logical indicating whether status information is printed during
 #'   the fitting process.
-#' 
+#'
 #' @details
 #' \describe{
 #'   \item{Response Types:}{\code{binary}, \code{numeric}, \code{Surv}}
@@ -21,24 +21,24 @@
 #'     \code{mstop}
 #'   }
 #' }
-#' 
+#'
 #' Default values for the \code{NULL} arguments and further model details can be
 #' found in the source links below.
-#' 
+#'
 #' @return \code{MLModel} class object.
-#' 
+#'
 #' @seealso \code{\link[mboost]{glmboost}}, \code{\link[mboost]{Family}},
-#' \code{\link{fit}}, \code{\link{resample}}, \code{\link{tune}}
-#' 
+#' \code{\link{fit}}, \code{\link{resample}}
+#'
 #' @examples
 #' library(MASS)
-#' 
+#'
 #' fit(type ~ ., data = Pima.tr, model = GLMBoostModel)
 #'
 GLMBoostModel <- function(family = NULL, mstop = 100, nu = 0.1,
                           risk = c("inbag", "oobag", "none"),
                           stopintern = FALSE, trace = FALSE) {
-  
+
   args <- params(environment())
   is_main <- names(args) %in% "family"
   params <- args[is_main]
@@ -48,7 +48,8 @@ GLMBoostModel <- function(family = NULL, mstop = 100, nu = 0.1,
     name = "GLMBoostModel",
     label = "Gradient Boosting with Linear Models",
     packages = "mboost",
-    response_types = c("binary", "numeric", "Surv"),
+    response_types = c("binary", "BinomialVariate", "NegBinomialVariate",
+                       "numeric", "PoissonVariate", "Surv"),
     predictor_encoding = "terms",
     params = params,
     grid = function(x, length, ...) {
@@ -59,9 +60,12 @@ GLMBoostModel <- function(family = NULL, mstop = 100, nu = 0.1,
     fit = function(formula, data, weights, family = NULL, ...) {
       if (is.null(family)) {
         family <- switch_class(response(data),
-                               "factor" = mboost::Binomial(),
-                               "numeric" = mboost::Gaussian(),
-                               "Surv" = mboost::CoxPH())
+                               BinomialVariate = mboost::Binomial(type = "glm"),
+                               factor = mboost::Binomial(),
+                               NegBinomialVariate = mboost::NBinomial(),
+                               numeric = mboost::Gaussian(),
+                               PoissonVariate = mboost::Poisson(),
+                               Surv = mboost::CoxPH())
       }
       eval_fit(data,
                formula = mboost::glmboost(formula, data = as.data.frame(data),
@@ -74,10 +78,9 @@ GLMBoostModel <- function(family = NULL, mstop = 100, nu = 0.1,
     predict = function(object, newdata, times, ...) {
       newdata <- as.data.frame(newdata)
       if (object$family@name == "Cox Partial Likelihood") {
-        y <- object$response
         lp <- drop(predict(object, type = "link"))
         new_lp <- drop(predict(object, newdata = newdata, type = "link"))
-        predict(y, lp, times, new_lp, ...)
+        predict(object$response, lp, times, new_lp, ...)
       } else {
         predict(object, newdata = newdata, type = "response")
       }
@@ -86,7 +89,7 @@ GLMBoostModel <- function(family = NULL, mstop = 100, nu = 0.1,
       structure(mboost::varimp(object), class = "numeric")
     }
   )
-  
+
 }
 
 MLModelFunction(GLMBoostModel) <- NULL
