@@ -1,24 +1,29 @@
 #' Linear Models
 #'
 #' Fits linear models.
-#' 
+#'
 #' @details
 #' \describe{
 #'   \item{Response Types:}{\code{factor}, \code{matrix}, \code{numeric}}
 #' }
-#' 
+#'
 #' Further model details can be found in the source link below.
-#' 
+#'
+#' In calls to \code{\link{varimp}} for \code{LModel}, numeric argument
+#' \code{base} may be specified for the (negative) logarithmic transformation of
+#' p-values [defaul: \code{exp(1)}].  Transformed p-values are automatically
+#' scaled in the calculation of variable importance to range from 0 to 100.  To
+#' obtain unscaled importance values, set \code{scale = FALSE}.
+#'
 #' @return \code{MLModel} class object.
-#' 
-#' @seealso \code{\link[stats]{lm}}, \code{\link{fit}}, \code{\link{resample}},
-#' \code{\link{tune}}
-#' 
+#'
+#' @seealso \code{\link[stats]{lm}}, \code{\link{fit}}, \code{\link{resample}}
+#'
 #' @examples
 #' fit(sale_amount ~ ., data = ICHomes, model = LMModel)
 #'
 LMModel <- function() {
-  
+
   MLModel(
     name = "LMModel",
     label = "Linear Model",
@@ -30,8 +35,7 @@ LMModel <- function() {
       y <- response(data)
       data <- as.data.frame(data)
       if (is.factor(y)) {
-        y_name <- deparse(response(formula))
-        formula[[2]] <- as.symbol(y_name)
+        y_name <- response(formula)
         if (nlevels(y) == 2) {
           y <- y == levels(y)[2]
           data[[y_name]] <- y
@@ -52,9 +56,11 @@ LMModel <- function() {
       newdata <- as.data.frame(newdata)
       predict(object, newdata = newdata)
     },
-    varimp = function(object, ...) varimp_wald(object)
+    varimp = function(object, base = exp(1), ...) {
+      varimp_pval(object, base = base)
+    }
   )
-  
+
 }
 
 MLModelFunction(LMModel) <- NULL
