@@ -48,10 +48,8 @@ VarImp.numeric <- function(object, ...) {
 #' @examples
 #' ## Survival response example
 #' library(survival)
-#' library(MASS)
 #'
-#' gbm_fit <- fit(Surv(time, status != 2) ~ sex + age + year + thickness + ulcer,
-#'                data = Melanoma, model = GBMModel)
+#' gbm_fit <- fit(Surv(time, status) ~ ., data = veteran, model = GBMModel)
 #' (vi <- varimp(gbm_fit))
 #' plot(vi)
 #'
@@ -92,6 +90,11 @@ varimp_pval.mlm <- function(object, ...) {
 }
 
 
+varimp_pval.multinom <- function(object, ...) {
+  varimp_pval(t(coef(object)), diag(vcov(object)), ...)
+}
+
+
 varimp_pval.numeric <- function(object, var, base = exp(1), ...) {
   -log(pchisq(object^2 / var, 1, lower.tail = FALSE), base = base)
 }
